@@ -28,6 +28,10 @@ from pdm.backend.utils import (
     to_filename,
 )
 
+SCHEME_NAMES = frozenset(
+    ["purelib", "platlib", "include", "platinclude", "scripts", "data"]
+)
+
 if sys.version_info < (3, 8):
     from importlib_metadata import version as get_version
 else:
@@ -72,6 +76,13 @@ class WheelBuilder(Builder):
         super().__init__(location, config_settings)
         self.__tag: str | None = None
 
+    def scheme_path(self, name: str, relative: str) -> str:
+        if name not in SCHEME_NAMES:
+            raise ValueError(
+                f"Unknown scheme name {name!r}, must be one of {SCHEME_NAMES}"
+            )
+        return f"{self.name_version}.data/{name}/{relative}"
+
     def _get_platform_tags(self) -> tuple[str | None, str | None, str | None]:
         python_tag: str | None = None
         py_limited_api: str | None = None
@@ -118,6 +129,22 @@ class WheelBuilder(Builder):
                 relpath = relpath[len(package_dir) + 1 :]
             yield relpath, path
         yield from self._get_metadata_files(context)
+        yield from self._get_wheel_data(context)
+
+    def _get_wheel_data(self, context: Context) -> Iterable[tuple[str, Path]]:
+        for name, paths in context.config.build_config.wheel_data.items():
+            for path in paths:
+                relative_to: str | None = None
+                if not isinstance(path, str):
+                    relative_to = path.get("relative-to")
+                    path = path["path"]
+                for child in context.expand_paths(path):
+                    relpath = (
+                        child.relative_to(relative_to).as_posix()
+                        if relative_to
+                        else child.name
+                    )
+                    yield self.scheme_path(name, relpath), child
 
     def build_artifact(
         self, context: Context, files: Iterable[tuple[str, Path]]
@@ -184,7 +211,7 @@ class WheelBuilder(Builder):
     def _write_dist_info(self, parent: Path) -> Path:
         """write the dist-info directory and return the path to it"""
         dist_info = parent / self.dist_info_name
-        dist_info.mkdir(0o700, parents=True, exist_ok=True)
+        dist_info.mkdir(0o700, exist_ok=True)
         meta = self.config.metadata
         entry_points = meta.entry_points
         if entry_points:
@@ -271,7 +298,7 @@ class WheelBuilder(Builder):
         if context.kwargs.get("metadata_directory"):
             return self._iter_files_in_directory(context.kwargs["metadata_directory"])
         else:
-            dist_info = self._write_dist_info(context.build_dir)
+            dist_info = self._write_dist_info(context.ensure_build_dir())
             return self._iter_files_in_directory(str(dist_info))
 
     def _iter_files_in_directory(self, path: str) -> Iterable[tuple[str, Path]]:
